' ========================================================================================
' Demonstrates how to set the compression and color depth when converting an image
' to the tiff format. The EncoderParameters structure has two members, a dword with
' the number of EncoderParameter structures and an array of EncoderParameter
' structures. As PB doesn't allow structures with dynamic arrays, an alternative is
' to build a buffer to hold the data and pass a pointer to this buffer to the
' GdipSaveImageToFile function.
' ========================================================================================

' SED_PBWIN
#COMPILE EXE
#DIM ALL
#INCLUDE "GDIPLUS.INC"
#INCLUDE "GDIPUTILS.INC"

' ========================================================================================
' Main
' ========================================================================================
FUNCTION PBMAIN

   LOCAL hStatus AS LONG
   LOCAL token AS DWORD
   LOCAL StartupInput AS GdiplusStartupInput
   LOCAL s AS STRING
   LOCAL EncoderClsid AS GUID
   LOCAL pImage AS DWORD
   LOCAL LoadFlName AS STRING
   LOCAL SaveFlName AS STRING
   LOCAL ep1 AS EncoderParameter
   LOCAL ep2 AS EncoderParameter
   LOCAL strEncoderParameters AS STRING
   LOCAL Compression AS LONG
   LOCAL ColorDepth AS LONG

   ' --> Change as needed
   LoadFlName = UCODE$("RYDER_Winona_01.jpg")
   SaveFlName = UCODE$("RYDER_Winona_01.tiff")

   ' Fill and EncoderParameter structure wuth the compression
   Compression = %EncoderValueCompressionLZW
   ep1.pGUID = $EncoderCompression
   ep1.NumberOfValues = 1
   ep1.dwType = %EncoderParameterValueTypeLong
   ep1.value = VARPTR(Compression)

   ' Fill and EncoderParameter structure wuth the color depth
   ColorDepth = 24  ' 24-bits color depth
   ep2.pGUID = $EncoderColorDepth
   ep2.NumberOfValues = 1
   ep2.dwType = %EncoderParameterValueTypeLong
   ep2.value = VARPTR(ColorDepth)

   ' Build an EncoderParameters structure using an string
   strEncoderParameters = MKDWD$(2) & ep1 & ep2

   ' Initialize GDI+
   StartupInput.GdiplusVersion = 1
   hStatus = GdiplusStartup(token, StartupInput, BYVAL %NULL)
   IF hStatus THEN
      MSGBOX "Error initializing GDI+"
      EXIT FUNCTION
   END IF

   ' Retrieve the encoder clsid
   s = GdiPlusGetEncoderClsid("image/tiff")
   IF s = "" THEN
      MSGBOX "Encoder not installed"
      GOTO Terminate
   END IF
   EncoderClsid = GUID$(s)

   ' Load image from file
   hStatus = GdipLoadImageFromFile(STRPTR(LoadFlName), pImage)
   IF hStatus THEN
      MSGBOX "GdipLoadImageFromFile - Failure, status = " & STR$(hStatus)
      GOTO Terminate
   END IF

   ' Convert and save the image to file
   IF pImage THEN
      hStatus = GdipSaveImageToFile(pImage, STRPTR(SaveFlName), EncoderClsid, BYVAL STRPTR(strEncoderParameters))
      IF hStatus THEN
         MSGBOX "GdipSaveImageToFile - Failure, status = " & STR$(hStatus)
      ELSE
         MSGBOX "Image converted"
      END IF
      ' Dispose the image
      hStatus = GdipDisposeImage(pImage)
   END IF

Terminate:

   ' Shutdown GDI+
   GdiplusShutdown token

END FUNCTION
' ========================================================================================
